/*
 * Copyright (c) 2011 by Intevation GmbH, Germany <info@intevation.de>
 *
 * This file is part of MXD2map.
 *
 * This program is free software under the LGPL (>=v2.1)
 * Read the file LICENCE.txt coming with the software for details
 * or visit http://www.gnu.org/licenses/ if it does not exist.
 *
 * MXD2map has been developed on behalf of the
 * Bundesamt fuer Seeschifffahrt und Hydrographie (BSH) in Hamburg
 * by Intevation GmbH.
 *
 * Authors:
 * Raimund Renkert <raimund.renkert@intevation.de>
 * Bjoern Schilberg <bjoern.schilberg@intevation.de>
 * Stephan Holl <stephan.holl@intevation.de>
 */

package de.intevation.mxd.reader;

import org.apache.log4j.Logger;

import com.esri.arcgis.display.ISymbol;
import com.esri.arcgis.display.IMarkerSymbol;
import com.esri.arcgis.display.CharacterMarkerSymbol;
import com.esri.arcgis.support.ms.stdole.Font;
import com.esri.arcgis.display.IRgbColor;
import com.esri.arcgis.display.RgbColor;

import org.w3c.dom.Element;
import java.awt.Color;
import java.io.IOException;

/**
 * Reads character marker symbol information.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class CharacterMarkerSymbolReader
extends AbstractSymbolReader {

    /**
     * The logger.
     */
    private static final Logger logger =
        Logger.getLogger(CharacterMarkerSymbolReader.class);

    /**
     * Private member.
     */
    private CharacterMarkerSymbol symbol;

    /**
     * Constructor with symbol.
     *
     * @param symbol The symbol used to display points.
     */
    public CharacterMarkerSymbolReader(ISymbol symbol)
    throws Exception {
        logger.debug("constructor()");
        if(symbol instanceof CharacterMarkerSymbol) {
            this.symbol = (CharacterMarkerSymbol)symbol;
        }
        else {
            throw new Exception("Not a CharacterMarkerSymbol!");
        }
    }

    /**
     * Constructor with symbol.
     *
     * @param symbol The symbol used to display points.
     */
    public CharacterMarkerSymbolReader(IMarkerSymbol symbol)
    throws Exception {
        logger.debug("constructor()");
        if(symbol instanceof CharacterMarkerSymbol) {
            this.symbol = (CharacterMarkerSymbol)symbol;
        }
        else {
            throw new Exception("Not a CharacterMarkerSymbol!");
        }
    }

    /**
     * Reads the symbol attributes.
     *
     * @return The XML node.
     */
    public Element read() {
        logger.debug("read()");
        Element symbolElement = util.addSymbol(parent);

        // Save the rotationField value of the Element
        if (!parent.getAttribute("rotationField").isEmpty()) {
            logger.debug("Setting angleBinding to rotationField: " +
                    parent.getAttribute("rotationField"));
            symbolElement.setAttribute("angleBinding",
                    parent.getAttribute("rotationField"));
        }

        try {
            symbolElement.setAttribute(
                "angle",
                String.valueOf(symbol.getAngle()));
        }
        catch(IOException ioe) {
            logger.warn("Could not read angle.");
        }

        try {
            symbolElement.setAttribute(
                "size",
                String.valueOf(symbol.getSize()));
        }
        catch(IOException ioe) {
            logger.warn("Could not read size.");
        }

        try {
            symbolElement.setAttribute(
                "x_offset",
                String.valueOf(symbol.getXOffset()));
        }
        catch(IOException ioe) {
            logger.warn("Could not read x-offset.");
        }

        try {
            symbolElement.setAttribute(
                "y_offset",
                String.valueOf(symbol.getYOffset()));
        }
        catch(IOException ioe) {
            logger.warn("Could not read y-offset.");
        }

        try {
            symbolElement.setAttribute(
                "name",
                symbol.getNameString());
        }
        catch(IOException ioe) {
            logger.warn("Could not read name. Setting name to \"default\"");
            symbolElement.setAttribute("name", "default");
        }

        try {
            symbolElement.setAttribute(
                "char",
                String.valueOf(symbol.getCharacterIndex()));
        }
        catch(IOException ioe) {
            logger.warn(
                "Could not read character number.");
        }

        try {
            if(symbol.getColor() instanceof IRgbColor) {
                IRgbColor color = (IRgbColor)symbol.getColor();
                Color c = new Color (
                    color.getRed(),
                    color.getGreen(),
                    color.getBlue());
                symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
                symbolElement.setAttribute("transparency",
                    String.valueOf(color.getTransparency()));
            }
            else {
                RgbColor col = new RgbColor();
                col.setRGB(symbol.getColor().getRGB());
                Color c = new Color (
                    col.getRed(),
                    col.getGreen(),
                    col.getBlue());
                symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
                symbolElement.setAttribute("transparency",
                    String.valueOf(col.getTransparency()));
            }
        }
        catch(IOException ioe) {
            logger.warn("Could not read color.");
        }

        try {
            Font f = symbol.getFont();
            symbolElement.setAttribute("font", f.getName());
            symbolElement.setAttribute("char_set", String.valueOf(f.getCharset()));
            symbolElement.setAttribute("bold", String.valueOf(f.getBold()));
            symbolElement.setAttribute("italic", String.valueOf(f.getItalic()));
            symbolElement.setAttribute("font_size", String.valueOf(f.getSize()));
            symbolElement.setAttribute(
                "strike_through",
                String.valueOf(f.getStrikethrough()));
            symbolElement.setAttribute("weight", String.valueOf(f.getWeight()));
        }
        catch(IOException ioe) {
            logger.warn("Could not read font. Setting font to \"FreeSans\".");
            symbolElement.setAttribute("font", "FreeSans");
        }
        symbolElement.setAttribute("style", "char");
        symbolElement.setAttribute("type", "marker");
        return symbolElement;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
